@@ -1,5 +1,5 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 from trytond.model import ModelView, ModelSQL, fields
 from trytond.wizard import Wizard, StateView, Button
 from trytond.transaction import Transaction
@@ -87,18 +87,14 @@ class BOMInput(ModelSQL, ModelView):
 
     @fields.depends('product', 'uom')
     def on_change_product(self):
-        res = {}
         if self.product:
             uoms = self.product.default_uom.category.uoms
             if (not self.uom or self.uom not in uoms):
-                res['uom'] = self.product.default_uom.id
-                res['uom.rec_name'] = self.product.default_uom.rec_name
-                res['unit_digits'] = self.product.default_uom.digits
+                self.uom = self.product.default_uom
+                self.unit_digits = self.product.default_uom.digits
         else:
-            res['uom'] = None
-            res['uom.rec_name'] = ''
-            res['unit_digits'] = 2
-        return res
+            self.uom = None
+            self.unit_digits = 2
 
     @fields.depends('product')
     def on_change_with_uom_category(self, name=None):
@@ -227,7 +223,7 @@ class OpenBOMTree(Wizard):
     start = StateView('production.bom.tree.open.start',
         'production.bom_tree_open_start_view_form', [
             Button('Cancel', 'end', 'tryton-cancel'),
-            Button('Ok', 'tree', 'tryton-ok', True),
+            Button('OK', 'tree', 'tryton-ok', True),
             ])
     tree = StateView('production.bom.tree.open.tree',
         'production.bom_tree_open_tree_view_form', [
